if defined?(::HTTPClient)
  class HTTPClient
    private
    alias_method :orig_do_get_block, :do_get_block

    def do_get_block(req, proxy, conn, &block)
      retryable_response = nil
      bm = Benchmark.realtime do
        begin
          orig_do_get_block(req, proxy, conn, &block)
        rescue RetryableResponse => e
          retryable_response = e
        end
      end

      res = conn.pop
      headers = res.headers
      post_params = req.body
      post_params = Rack::Utils.parse_nested_query post_params if post_params.class == String

      Susu.log_all(
        req.header.request_method, req.header.request_uri,
        res.status_code, bm, post_params, res.body, headers,
        headers['Content-Encoding'], headers['Content-Type']
      )
      conn.push(res)

      raise retryable_response if retryable_response != nil
    end

    class Session
      alias_method :orig_create_socket, :create_socket

      # up to version 2.6, the method signature is `create_socket(site)`; after that,
      # it's `create_socket(hort, port)`
      if instance_method(:create_socket).arity == 1
        def create_socket(site)
          Susu.log_connection(site.host, site.port)
          # end
          orig_create_socket(site)
        end

      else
        def create_socket(host, port)
          Susu.log_connection(host, port)
          # end
          orig_create_socket(host,port)
        end
      end
    end
  end
end
